"""
Preps function to poccess clone in Async
"""
import asyncio
import logging
import threading

from merakitools import const, lib, model
from .processor import validate_network_processor

class Validateorginization(threading.Thread):
	"""
    Setups the orginization objet to run through clone and validate proccess
    """
	
	def __init__(self, org_id):
		"""
        Init Function or Validateorginization
        Args:
            org_id(string): Orginization ID from Meraki
        """
		threading.Thread.__init__(self)
		self.org_id = org_id
		self.name = model.meraki_nets[org_id].name
		self.change_log = []
	
	def run(self):
		"""
        Start the ASYNC fun fuctions and waits for completions to restore
        Cache
        Returns:

        """
		self.change_log = asyncio.run(lib.get_change_log_from_org(self.org_id))
		#golden_change_last_ts = asyncio.run(lib.get_network_last_change_ts(model.golden_nets[const.appcfg.tag_golden].networks[const.appcfg.tag_golden].org_id
		                                                                   #,model.golden_nets[const.appcfg.tag_golden].networks[const.appcfg.tag_golden].net_id))
		#is_golden_changed = asyncio.run(lib.check_last_change(model.golden_nets[const.appcfg.tag_golden].networks[const.appcfg.tag_golden])
		asyncio.run(self._async_run())
		asyncio.run(lib.update_change_log(self.org_id))
		lib.store_cache(self.org_id, model.meraki_nets[self.org_id],'autosync')
	
	async def _async_run(self):
		"""
        Async version of the run function loops through all networks
        then sends the network to the valate proccessor
        Returns:

        """
		print(
				f'\tOrgName: {self.name}Thread PID:{threading.currentThread().native_id}'
		)
		threading.currentThread().setName(self.name)
		print(f'\tThread Name:{threading.currentThread().name}')
		with lib.MerakiAsyncApi() as sdk:
			logger = logging.getLogger('meraki.aio')
			logger.setLevel(logging.WARNING)
			net_compare_task = [
					validate_network_processor(self.org_id, net_id, sdk)
					for net_id in model.meraki_nets[self.org_id].networks
					if const.appcfg.tag_golden not in model.meraki_nets[
						self.org_id].networks[net_id].tags
			]
			
			await asyncio.gather(*net_compare_task)


from merakitools import const,lib

class Devices:
	def __init__(self,):
		self.devices = []
		
	async def getDevices(self):
		with lib.MerakiAsyncApi*() as sdk:
			self.devices = sdk.organizations.getOrganizationInventoryDevices()


import time
import asyncio
import sys
import pandas as pd
from datetime import datetime
from merakitools import const, lib, model, utils,mnetutils


from tabulate import tabulate
def completioninfo():
	"""
    Prints table of infomration after all tasks are complete
    Args:
    Returns:

    """
	orgcount = 0
	network_count = 0
	output_table = []
	for org in model.meraki_nets:
		orgcount = orgcount + 1
		network_count = network_count + int(len(model.meraki_nets[org].networks))
		output_table.append({
				'Orginization Name': model.meraki_nets[org].name,
				'Total Network'    : len(model.meraki_nets[org].networks),
				'Sync Runtime'     : model.meraki_nets[org].syncruntime,
				'Last Sync'        : model.meraki_nets[org].lastsync
		})
	print(f'Total Orgs Synced: {orgcount} '
	      f'Total Network Synced: {network_count}')
	table = pd.DataFrame.from_dict(output_table)
	print(tabulate(table, headers='keys', tablefmt='psql'))
async def sync_task():
	for tags in const.appcfg.network_tags:
		model.meraki_nets = {}
		model.golden_nets = {}
		const.appcfg.tag_golden = tags['golden']
		const.appcfg.tag_target = tags['target']
		start_time = time.perf_counter()
		print(f'{lib.bc.OKBLUE} Starting sync for Golden-Tag: {tags["golden"]} to Target-Tag: {tags["target"]} at {start_time:0.5f} {lib.bc.ENDC}')
	
		model.golden_nets.update({
				const.appcfg.tag_golden:
					model.ORGDB(const.appcfg.tag_golden,
					            const.appcfg.tag_golden)
		})
		model.golden_nets[const.appcfg.tag_golden].networks.update(
				{const.appcfg.tag_golden: const.appcfg.tag_golden})
		if const.appcfg.use_cache:
			lib.load_cache(const.appcfg.tag_golden, model.golden_nets[const.appcfg.tag_golden], 'autosync')
		await mnetutils.setup_org_data_model()
		org_threads = []
		validate_thread = []
		for org_id in model.meraki_nets:
			org_threads.append(utils.Orgsyncprocessor(org_id))
		for orgthread in org_threads:
			orgthread.start()
		for orgthread in org_threads:
			orgthread.join()
		
		for org in model.meraki_nets:
			validate_thread.append(utils.Validateorginization(org))
		for thread in validate_thread:
			thread.start()
		for thread in validate_thread:
			thread.join()
		
		if const.appcfg.use_cache:
			model.golden_nets[
				const.appcfg.tag_golden].lastsync = datetime.utcnow()
			lib.store_cache(const.appcfg.tag_golden, model.golden_nets[
				const.appcfg.tag_golden], 'autosync')
		
		print(f'Total Job Runtime: '
		      f'{(time.perf_counter() - start_time):0.5f} secounds')
		completioninfo()



"""
Helper Functions Fo Device Sync, COnfiguration ETC
"""
import json
import http3
import requests
import asyncio
from merakitools import const, model
from datetime import datetime
from .bcolors import bcolors as bc
from .merakiapi import MerakiAsyncApi


async def aironetie(net_id: str, ssid_id: int):
    """
    Fuction to Looking up if aironet IE is configured on SSID
    Args:
        net_id: Netwokr ID
        ssid_id: SSID ID

    Returns:

    """
    try:
        client = http3.AsyncClient()
        response = await client.get(
                'https://api.meraki.com/api/v1/networks/%s/wireless/ssids/%s/overrides'
                % (net_id, ssid_id),
                headers={
                        'X-Cisco-Meraki-API-Key':
                            const.appcfg.MERAKI_DASHBOARD_API_KEY,
                        'Content-Type'          : 'application/json'
                })
        
        if response.status_code != requests.codes.ok:
            return 'null'
        return response.json()
    except Exception as error:
        if const.appcfg.debug:
            print(error)
        return "null"


async def set_aironet_ie(net_id: str, ssid_id: int, p_data: dict):
    # looks up org net_id for a specific org name
    # on failure returns 'null'
    """
    Sets Aironet IE on a spefic Network and SSID
    Args:
        net_id: Network ID
        ssid_id: SSID Number/ID
        p_data: Data for Configuration

    Returns:

    """
    response = requests.put(
            'https://api.meraki.com/api/v1/networks/%s/wireless/ssids/%s/overrides'
            % (net_id, ssid_id),
            data=json.dumps(p_data),
            headers={
                    'X-Cisco-Meraki-API-Key': const.appcfg.MERAKI_DASHBOARD_API_KEY,
                    'Content-Type'          : 'application/json'
            })
    if response.status_code != requests.codes.ok:
        return 'null'
    
    return response.json()


async def rfp_pwr(rf_power):
    """
    RF Power Helper function to check if min and max power ssettings will fail
    Args:
        rf_power: RF Power Settings

    Returns: RF Power Settings DICT
    """
    
    if 'twoFourGhzSettings' in rf_power:
        if 'minPower' in rf_power['twoFourGhzSettings'] and rf_power[
            'twoFourGhzSettings']['minPower'] < 5:
            rf_power['twoFourGhzSettings']['minPower'] = 5
        if 'maxPower' in rf_power['twoFourGhzSettings'] and rf_power[
            'twoFourGhzSettings']['maxPower'] < 5:
            rf_power['twoFourGhzSettings']['maxPower'] = 5
    
    if 'fiveGhzSettings' in rf_power:
        if 'minPower' in rf_power['fiveGhzSettings'] and rf_power[
            'fiveGhzSettings']['minPower'] < 5:
            rf_power['fiveGhzSettings']['minPower'] = 8
        if 'maxPower' in rf_power['fiveGhzSettings'] and rf_power[
            'fiveGhzSettings']['maxPower'] < 5:
            rf_power['fiveGhzSettings']['maxPower'] = 8
    return rf_power


def print_update(net_name: str, task: str, product: str):
    """
    Prints the update message to screen
    Args:
        net_name: Network Name
        task: Current Task
        product: Current Product

    Returns:

    """
    print(f'\t {bc.FAIL} {net_name} {bc.WARNING} - Updating Settings {str(product).upper()} '
          f'{str(task).upper()}..{bc.ENDC}')


def print_matched(net_name: str, task: str, product: str):
    """
    Print Settings matched statment
    Args:
        net_name: Network Name
        task: Current Running Task
        product: Current Product

    Returns:
    """
    if const.appcfg.enable_status:
        print(f'\t{bc.OKBLUE} {net_name} {bc.OKGREEN}'
            f'- Settings for {str(product).upper()} - {str(task).upper()} Matched Golden Network {bc.ENDC}')


def get_golden(product):
    """
    Gets Golden Configuration per Product and returns to calling function
    Args:
        product: Product that Config Is need

    Returns: Golden Config Object for Product

    """
    golden_tag = const.appcfg.tag_golden
    return model.golden_nets[golden_tag].networks[golden_tag].dashboard[
        product]





def rf_profile_pre_proccess(profiles):
    """
    Perpars the rf profile object for compairing by removing items that will always ne diofferemt
    ;like rf profileID  and networkID   Also remvoes the valid channels for 5GHz due to a issue with the current API
    For now valid RF channles will be stored in the Application Configuration file
    Args:
        profiles(list): RF Profile Object

    Returns(list): List of dicts of RF Prolfies

    """
    for profile in profiles:
        if 'validAutoChannels' in profile['fiveGhzSettings']:
            profile['fiveGhzSettings'].pop(
                    'validAutoChannels')
        if 'id' in profile:
            profile.pop('id')
        if 'networkId' in profile:
            profile.pop('networkId')
    return profiles





def add_valid_channels(profile: dict):
    """
    Adds 5Ghz channels back into the profile for update or creates
    this is a temp fix unitl
    Args:
        profile(dict): RF Profile Dict

    Returns: RF Profile

    """
    profile['fiveGhzSettings'][
        'validAutoChannels'] = const.appcfg.ffive_ghz_valid_channcles
    return profile



async def get_network_last_change_ts(sdk, org_id, net_id):
    change_log = await sdk.organizations.getOrganizationConfigurationChanges(
            org_id, networkId=net_id)
    return datetime.fromisoformat(str(change_log[0]['ts']).strip('Z'))


async def update_cached_change_ts(change_log, org_id, net_id):
    net_ts = list(filter(lambda nts: nts['networkId'] in net_id,
                         [ts for ts in change_log if
                          'networkId' in ts.keys()]))
    model.meraki_nets[org_id].networks[
        net_id].change_log_ts = datetime.fromisoformat(
        str(net_ts[0]['ts']).strip('Z'))


async def get_change_log_from_org(org_id):
    with MerakiAsyncApi() as sdk:
        change_log = await sdk.organizations.getOrganizationConfigurationChanges(
            org_id)
    return change_log

async def update_change_log(org_id):

    change_log = await get_change_log_from_org(org_id)
    model.meraki_nets[org_id].change_log = change_log
    change_log_task = [
            update_cached_change_ts(change_log,org_id, net_id) for
            net_id in model.meraki_nets[org_id].networks]
    await asyncio.gather(*change_log_task)

async def last_change(change_log,net_id):
    net_ts = list(filter(lambda nts: nts['networkId'] in net_id,
                         [ts for ts in change_log if 'networkId' in ts.keys()]))
    return datetime.fromisoformat(str(net_ts[0]['ts']).strip('Z'))

def build_approved_config(port_config,tag):
    config = {}
    for config_item in port_config:
        if config_item in const.appcfg.switch_port_tags[tag]:
            config.update({config_item: port_config[config_item]})
    return config


"""
Application configuration model
Holds all appliaction configuration infomation to be used between different
methods
"""
import csv
import json
import sys
from datetime import timedelta
from distutils.util import strtobool
from logging import getLevelName
from os import getenv, mkdir, path


# noinspection PyBroadException
class APPCONFIG:
    """
    Appliction Configuration Object
    """

    def __init__(self, file=None):
        self.tag_target = None
        self.tag_golden = None
        if file is not None:
            self.psk_data = []
            if str(file).endswith('.json'):
                self._open_config(self.getdirectory(file))
            elif isinstance(file, dict):
                self._load_from_env(True)
                self._config_from_file(file)
            else:
                print('Incorrect File Format')
        else:
            self._load_from_env()

    def _read_csv(self):
        if self.usepsk_file:
            try:
                with open(self.getdirectory(self.psk_file), 'r',
                          encoding='utf-8-sig') as file:
                    for row in csv.DictReader(file, delimiter=','):
                        self.psk_data.append(row)
            except Exception as error:
                print(f'Error reading PSK CSV File {error}')

    def _open_config(self, file):
        with open(file, 'r') as read_file:
            _config = json.loads(read_file.read())
            self._config_from_file(_config)

    def _config_from_file(self, config):
        for item in config:
            if config[item] is None:
                continue
            setattr(self, item, config[item])
        self.cache_dir = self.getdirectory(self.cache_dir)
        self.log_path = self.getdirectory(self.log_path)
        self.psk_file = self.getdirectory(self.psk_file)
        self.logging_level = getLevelName(str(self.logging_level).upper())
        if 'MERAKI_DASHBOARD_API_KEY' not in self.__dict__.keys() \
                or self.MERAKI_DASHBOARD_API_KEY == 'None' \
                or self.MERAKI_DASHBOARD_API_KEY == "null":
            self.MERAKI_DASHBOARD_API_KEY = getenv('MERAKI_DASHBOARD_API_KEY',
                                                   None)
            if self.MERAKI_DASHBOARD_API_KEY is None \
                    or self.MERAKI_DASHBOARD_API_KEY == "null":
                print(
                    'Please add meraki_dashboard_api_key by running '
                    '"autosync config setkey <API Key>"'
                    'or add meraki_dashboard_api_key: <API Key > to your config.json file'
                )
                sys.exit()
        if self.usepsk_file and (self.psk_file is not None):
            self._read_csv()

    def _load_from_env(self, override=False):
        self.use_env = True
        self.MERAKI_DASHBOARD_API_KEY = getenv('meraki_dashboard_api_key',
                                               None)
        if self.meraki_dashboard_api_key is None:
            if override:
                self.meraki_dashboard_api_key = "null"
            else:
                print(
                    'Please Set meraki_dashboard_api_key by running '
                    'autosync config setkey to contunue  or add it to the .env file '
                )
                sys.exit()
        self.meraki_base_url = getenv('meraki_base_url',
                                      'https://api.meraki.com/api/v1/')
        self.simulate = bool(strtobool(getenv('simulate', 'False')))
        self.wait_on_rate_limit = bool(
            strtobool(getenv('wait_on_rate_limit', 'True')))
        self.maximum_concurrent_requests = int(getenv(
            'maximum_concurrent_requests', "3"))
        self.nginx_429_retry_wait_time = int(getenv('nginx_429_retry_wait_time',
                                                    "8"))
        self.maximum_retries = int(getenv('maximum_retries', "100"))
        self.log_path = self.getdirectory(getenv('log_path', '~/Logs'))
        self.suppress_logging = bool(
            strtobool(getenv('suppress_logging', 'False')))
        # Set this to FALSE for READ-ONLY, TRUE for "R/W"
        self.write = bool(strtobool(getenv('write', 'False')))
        # Set this to true, to crawl all networks. WARNING. Don't set write
        # & all_orgs unless you know what you're doing and dislike your job
        self.all_orgs = bool(strtobool(getenv('all_orgs', 'False')))
        if self.all_orgs:
            self.allow_org_list = []
        else:
            ## Only monitor these orgs, to keep the "crawl" down
            try:
                self.allow_org_list = list(getenv('allow_org_list').split(','))
                self.allow_org_list_name = list(getenv('allow_org_list_name').split(','))
            except Exception:
                self.allow_org_list = []
            try:
                self.allow_org_list_name = list(getenv('allow_org_list_name').split(','))
            except Exception:
                self.allow_org_list_name = []
        # Include switch settings?
        self.switch = bool(strtobool(getenv('switch', 'True')))
        # TARGET should be on ALL networks that are inscope, the master and
        # all the target networks
        self.tag_target = list(eval(getenv('network_tags', '[{"golden":"autosync"}]')))
        # MASTER should ONLY be on the 'golden network'
        self.tag_golden = getenv('tag_golden', 'master')
        # USed for Scale Testing Development
        self.tag_override = bool(strtobool(getenv('tag_override', 'False')))
        self.open_roaming = getenv('open_roaming', 'Meraki123')
        self.cache_dir = self.getdirectory(
            str(getenv('CHACHE_DIR', '~/mnetCache')))
        self.cache_timeout = int(getenv('cache_timeout', '24'))
        self.use_cache = bool(strtobool(getenv('use_cache', 'True')))
        self.rad_keys_all = getenv('rad_keys_all', 'Meraki123')
        # noinspection PyBroadException
        try:
            self.ssid_skip_psk = list(
                getenv('ssid_skip_psk', 'SSID1,SSID2').split(','))
        except:
            self.ssid_skip_psk = []
        self.logging_level = getLevelName(
            str(getenv('logging_level', "ERROR")).upper())
        self.target_vlan = bool(strtobool(getenv('target_vlan', 'True')))
        self.debug = bool(strtobool(getenv('debug', 'False')))
        self.override_psk = getenv('override_psk', None)
        self.usepsk_file = bool(strtobool(getenv('usepsk_file', 'False')))
        if self.usepsk_file:
            self.psk_file = self.getdirectory(str(getenv('psk_file', None)))
            self.psk_file_net_name = str(getenv('psk_file_net_name', 'RackCode'))
            self.psk_file_psk = str(getenv('psk_file_psk', 'PSK'))
            if self.psk_file is not None:
                self._read_csv()
        self.five_ghz_valid_channcles = list(getenv('five_ghz_valid_channcles',
                                                    '36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108,112, 136, 140, 144, 153, 157, 161').split(
            ','))
        self.enable_status = bool(strtobool(getenv('enable_status', 'True')))


    def getdirectory(self,location, cfg_file=False):
        """
        Gets OS Pation of working direcoty
        Args:
            location: File location as string
            cfg_file: Is Config Files

        Returns: Return full direcotry of string in file system

        """
        if location is None:
            return None
        try:
            if str(location).startswith('~/'):
                if not path.exists(path.expanduser(location)):
                    if str(location).endswith('.json'):
                        print('File Not Found')
                        if cfg_file:
                            sys.exit()
                    else:
                        mkdir(path.expanduser(location))
                return path.expanduser(location)
            else:
                if not path.exists(path.abspath(location)):
                    if str(location).endswith('.json'):
                        print('File Not Found')
                        if cfg_file:
                            sys.exit()
                    else:
                        mkdir(path.abspath(location))
                return path.abspath(location)
        except Exception as error:
            if self.usepsk_file:
                print(f'Error with PSK File {error}')
                sys.exit()
            else:
                pass

    def dump_config_to_file(self):
        """
        Dumps Configuration to file for reuse later
        Returns:

        """
        with open('config.json', 'w+') as file:
            config = self.__dict__
            print(json.dumps(config, indent=4, sort_keys=True))
            file.write(json.dumps(config, indent=4, sort_keys=True))

    @staticmethod
    def get_rad_sec():
        """
        Possible not used anymore
        Returns: Radius Sec
        """
        return getenv('rad_keys_all')

    def check_cache(self):
        """
        Chaceks time out cache
        Returns: time delta

        """
        delta = timedelta(hours=self.cache_timeout)
        return delta



import os
import click
from merakitools import lib

@click.group()
def cli():
	""" Export of Cache MNET datafile to JSON Formated file """
	pass


@click.command(help="Exports MNET Files to JSON Formted")
@click.option('-i', '--cachefile', required=True,
              help='Full Path and file name to .mnet file EX: /opt/dat/cache/golden/MyOrg-1223431.mnet')
@click.option('-d', '--outputdir', required=True,
              help='Output direcotry for JSON file EX: ~/output')
def json(cachefile,outputdir):
	lib.dump_cache_file_to_json(cachefile,outputdir)


cli.add_command(json)



"""
    Main Module of autosync Application
"""
import asyncio
from merakitools import meraki_tasks

from merakitools import const, lib, model, utils
from merakitools.model import golden_nets, meraki_nets


def setup_app(cfg_file=None):
	const.appcfg = model.APPCONFIG(cfg_file)
	temp_sdk = lib.MerakiApi()
	const.meraki_sdk = temp_sdk.api


def run(cfg_file=None,task='sync'):
	"""
    Main Module Start of Application
    Args:
    Returns:

    """
	setup_app(cfg_file)
	if task == 'sync':
		asyncio.run(meraki_tasks.sync_task())
	elif task == 'device_config':
		asyncio.run(meraki_tasks.device_config())
	
	

if __name__ == '__main__':
	config_file = '~/autosync/config.json'
	run(config_file,"device_config")
	print('Done')



"""
Functions for loading, storing and clearing of Cache Files
This can also be used for backup function in future releases
"""
import pickle
import jsonpickle
import json
from datetime import datetime
from os import path, remove, mkdir
from merakitools import const, model


def cache_age(item):
    """
	Determins age of Cache file
	Args:
		item: Data Model item that has the last_sync value

	Returns: time delta from time utc now to the value stored in last+sync

	"""
    return datetime.utcnow() - item.last_sync


# returns the cached object if exists otherwise returns the locally synced
def load_cache(org_id: str, org, task: str):
    """
	Lods Cache into Memory from stored pickle files
	Args:
		is_golden: If Master True Load org from Master Dict
		org_id: Org ID of meraki Org to load
		task: Current Task to store cache
	Returns:

	"""

    full_path = check_and_create_dir(const.appcfg.cache_dir, task,
                                     const.appcfg.tag_golden)
    cache_file_str = f'{full_path}/{str(org.org_id)}-{str(org.name)}.mnet'
    if path.exists(cache_file_str) and const.appcfg.use_cache:
        with open(cache_file_str, "rb") as cache_file:
            org_cached = pickle.load(cache_file)
        if org_cached.lastsync is None:
            print('Has Cache! But it is stale, re-syncing')
            clear_cache(org_id, org, 'autosync')
            org.cached = False
        elif datetime.utcnow() - datetime.fromisoformat(
                str(org_cached.lastsync)):
            org.__dict__ = org_cached.__dict__.copy()
        else:
            print('Cache File Error! But it is stale, re-syncing')
            clear_cache(org_id, org, 'autosync')
            org.cached = False
    else:
        print('No Cache Found')
        org.cached = False


# writes it to disk for faster load times
def store_cache(org_id: str, org, task: str):
    """
	Stores Cache to file system as pickle files
	Args:
		org_id: Orginixation ID
		is_golden (object): Golden Network
		task: Current Running Task

	Returns:

	"""
    if not const.appcfg.use_cache:
        return
    
    org.cached = True

    clear_cache(org_id, org, 'autosync')
    full_path_str = check_and_create_dir(const.appcfg.cache_dir, task,
                                         const.appcfg.tag_golden)
    cache_file = f'{full_path_str}/{str(org.org_id)}-{str(org.name)}.mnet'
    pickle.dump(org, open(cache_file, "wb"))


# clears cache and kills disk backup
def clear_cache(org_id: str, org, task: str):
    """
	Clears Cache ( Delets Pickle file from File System
	Args:
		org_id: Orginixation ID
		is_golden: Golden Network
		task: Current Running Task

	Returns:

	"""
    cache_file_str = \
     f'{const.appcfg.cache_dir}/{task}/{const.appcfg.tag_golden}/{str(org.org_id)}-{str(org.name)}.mnet'
    if path.exists(cache_file_str):
        remove(cache_file_str)


def _chk_dir_name(file: str):
    """
	Checks DIE name for home folder
	Args:
		file:

	Returns:ABS Path for fiolder

	"""
    if file.startswith('~/'):
        return path.expanduser(file)
    else:
        return path.abspath(file)


def check_and_create_dir(str_base_path: str, str_task: str, str_tag: str):
    if not path.exists(str_base_path):
        mkdir(_chk_dir_name(str_base_path))
    full_path = f'{str_base_path}/{str_task}'
    if not path.exists(full_path):
        mkdir(_chk_dir_name(full_path))
    full_path = f'{full_path}/{str_tag}'
    if not path.exists(full_path):
        mkdir(_chk_dir_name(full_path))
    return full_path


def dump_cache_file_to_json(cache_file: str, output_dir: str):
    """
	Dumps Pickle Cache File to a JSON file for Troubleshooting and validation
	Args:
		cache_file(STR): Cache Full Path and Name with extension
		output_dir(str): Direcotory to output cache file
		

	Returns:

	"""
    fpath, name = path.split(cache_file)
    output_file_name = f'{name.split(".")[0]}.json'
    cache_file = _chk_dir_name(cache_file)
    output_full_path = _chk_dir_name(f'{output_dir}/{output_file_name}')
    output_dir = _chk_dir_name(output_dir)
    print(f'Staring export of cache to JSON')
    if path.exists(cache_file):
        with open(cache_file, "rb") as cache_file:
            org_cached = pickle.load(cache_file)
        orgJSON = jsonpickle.encode(org_cached, unpicklable=False)
        output_json = json.loads(orgJSON)
        if not path.exists(output_dir):
            mkdir(output_dir)
        with open(output_full_path, 'w', encoding='utf-8') as dump_file:
            json.dump(output_json, dump_file, ensure_ascii=False, indent=4)
        print(
            f'Cache File {cache_file.name} exported to JSON file {output_full_path}'
        )
    else:
        print('Input cache file does not exisit')


if __name__ == '__main__':
    dump_cache_file_to_json(
        '~/mnetCache/622059698530550345-CiscoLab-Manhattan1.mnet',
        '~/apps/testSync/output')



"""
Data Class for meraki Devkices
"""

class Device:
	"""
	Data Model for Meraki Devices
	Args:
			serial: Serial number of device
			mac: Mac Address of ddevice
			net_id: Network Id device is located
			org_id: Org Id devices is located
			model: Model Number
			tags: Meraki tags assigned to device
	"""
	def __init__(self,device,org_id):
		"""
		Inits teh Data modle for Meraki Devices
		Args:
			device(dict): JSON response from the network device API call
			org_id(str): Current Orginization ID
		"""
		self.name = device['name']
		self.serial =  device['serial']
		self.mac = device['mac']
		self.net_id = device['networkId']
		self.org_id = org_id
		self.model = device['model']
		self.tags = device['tags']
		self.last_update = device['configurationUpdatedAt']
		self.prodect = None
		self._get_type()
		self.config = []
	
	def _get_type(self):
		if str(self.model).startswith('MR'):
			self.prodect = 'wireless'
		elif str(self.model).startswith('MS'):
			self.prodect = 'switch'


"""
Network Product Setup for all configuration items to be synced
"""
import copy

from meraki.exceptions import AsyncAPIError

from merakitools import const, lib
from merakitools.mnetutils.resync import re_sync


class wireless:
    """
	Wireless Settings and functions for the meraki Dashboard
	"""
    def __init__(self):
        # Dahsboard Wireless Settings
        self.NetworkWirelessSettings = None
        self.NetworkWirelessBluetoothSettings = None
        self.NetworkWirelessRfProfiles = None
        self.ssids_range = [
        ]  # should hold array of SSID_IDs, ex. [0,1,2,4,6,7]
        self.NetworkWirelessSsid = []
        self.NetworkWirelessSsidFirewallL3FirewallRules = []
        self.NetworkWirelessSsidFirewallL7FirewallRules = []
        self.NetworkWirelessSsidTrafficShapingRules = []
        self.NetworkWirelessSsidIdentityPsks = []
        self.overridePSK = None
        self.hasAironetIE = None
        self.aironetie = None
        self.NetworkName = None

    def settings(self):
        """
		List class object that we are storing data but don't need to run
		Returns:

		"""
        skip = ['re_sync', 'hasAironetIE', 'ssids_range', 'NetworkName']
        items = [i for i in self.__dict__.keys() if i not in skip]
        return items

    async def Get_overridePSK(self, sdk: object, net_id: str, net_name: str):
        """
		Gets Overrid PSK from CSV file defimd in appconfig
		Args:
			sdk: Meraki SDK
			net_id: Network ID
			net_name: Network Name

		Returns:
		"""
        if const.appcfg.debug:
            print(f'Network ID: {net_id} - {net_name}')
        found = False
        if const.appcfg.usepsk_file and (const.appcfg.psk_file is not None):
            for item in const.appcfg.psk_data:
                if item[const.appcfg.psk_file_net_name].upper() == str(
                        net_name).upper():
                    self.overridePSK = item[const.appcfg.psk_file_psk]
                    found = True
                    if const.appcfg.enable_status:
                        print(
                            f'\t\t\t{lib.bc.OKGREEN} {net_name} - Network PSK Found'
                        )
                    continue

        if not found:
            if const.appcfg.enable_status:
                print(
                    f'\t\t\t{lib.bc.WARNING} {self.NetworkName} - No PSK Found'
                )

    async def Get_NetworkWirelessSsid(self, sdk: object, net_id: str,
                                      net_name: str):
        """
		Gets wirless SSIDs from meraki dashbaord
		Args:
			sdk: Meraki SDK
			net_id: Network ID
			net_name: Network Name

		Returns:
		"""
        if const.appcfg.debug:
            print(f'Network ID: {net_id} - {net_name}')
        ssids = []
        dashabordssids = await sdk.wireless.getNetworkWirelessSsids(net_id)
        for ssid in dashabordssids:
            ssids.append(ssid)
            if "Unconfigured SSID" not in ssid[
                    'name'] and not ssid['number'] in self.ssids_range:
                self.ssids_range.append(ssid['number'])
        self.NetworkWirelessSsid = ssids
        self.ssids_range

    async def Get_aironetie(self, sdk: object, net_id: str, net_name: str):
        """
		Get's aironet IE settings from meraki dashboard
		Args:
			sdk: Meraki SDK
			net_id: Network ID
			net_name: Network Name

		Returns:
		"""
        if const.appcfg.debug:
            print(f'Network ID: {net_id} - {net_name}')
        if (self.hasAironetIE is None) or (not self.hasAironetIE):
            # print(f'Network {name} has aironetIE extensions!!!')
            self.hasAironetIE = False
        # print(f'Network {name} needs aironetIE NFO')
        else:
            self.hasAironetIE = True

        # only do the full refresh if it's been cloned, cloneFrom_MR will set the aironetie = None
        if self.hasAironetIE:
            self.aironetie = []
            for i in range(0, 15):
                if i in self.ssids_range:  # only query/refresh the active SSIDS
                    aie_code = await lib.aironetie(net_id, i)
                    if const.appcfg.debug:
                        print(
                            f'\t\t\t{lib.bc.OKBLUE}Detecting AIE for SSID['
                            f'{lib.bc.WARNING}{i}{lib.bc.OKBLUE}] '
                            f'Status[{lib.bc.WARNING}{aie_code}{lib.bc.OKBLUE}]{lib.bc.ENDC}'
                        )
                    self.aironetie.append(
                        aie_code)  # -1 for unkown, 0 for off, 1 for
            # self.aironetie
        # self.aironetie

    async def Get_NetworkWirelessSsidFirewallL3FirewallRules(
            self, sdk: object, net_id: str, net_name: str):
        """
		Gets Layer3 Firewall Rules From Meraki DashBoard
		Args:
			sdk: Meraki SDK
			net_id: Network ID
			net_name: Network Name

		Returns:
		"""
        if const.appcfg.debug:
            print(f'Network ID: {net_id} - {net_name}')
        ssids_l3 = []
        for ssid_num in range(0, 15):
            if ssid_num in self.ssids_range:
                ssids_l3.append(
                    await
                    sdk.wireless.getNetworkWirelessSsidFirewallL3FirewallRules(
                        net_id, ssid_num))
            else:
                ssids_l3.append([])
        self.NetworkWirelessSsidFirewallL3FirewallRules = ssids_l3

    async def Get_NetworkWirelessSsidFirewallL7FirewallRules(
            self, sdk: object, net_id: str, net_name: str):
        """
		Gets Layer 7 Firewall Rules from Meraki Dashbaord
		Args:
			sdk: Meraki SDK
			net_id: Network ID
			net_name: Network Name

		Returns:
		"""
        if const.appcfg.debug:
            print(f'Network ID: {net_id} - {net_name}')
        ssids_l7 = []
        for ssid_num in range(0, 15):
            if ssid_num in self.ssids_range:
                ssids_l7.append(
                    await
                    sdk.wireless.getNetworkWirelessSsidFirewallL7FirewallRules(
                        net_id, ssid_num))
            else:
                ssids_l7.append([])
        self.NetworkWirelessSsidFirewallL7FirewallRules = ssids_l7

    async def Get_NetworkWirelessSsidTrafficShapingRules(
            self, sdk: object, net_id: str, net_name: str):
        """
		Gets Trafffic Shaping rules from meraki dashboard
		Args:
			sdk: Meraki SDK
			net_id: Network ID
			net_name: Network Name

		Returns:
		"""
        if const.appcfg.debug:
            print(f'Network ID: {net_id} - {net_name}')
        ssids_ts = []
        for ssid_num in range(0, 15):
            if ssid_num in self.ssids_range:
                ssids_ts.append(
                    await
                    sdk.wireless.getNetworkWirelessSsidTrafficShapingRules(
                        net_id, ssid_num))
            else:
                ssids_ts.append([])
        self.NetworkWirelessSsidTrafficShapingRules = ssids_ts

    async def Get_NetworkWirelessSsidIdentityPsks(self, sdk: object,
                                                  net_id: str, net_name: str):
        """
		GGets Idenity PSK from Meraki Dashboard
		Args:
			sdk: Meraki SDK
			net_id: Network ID
			net_name: Network Name

		Returns:
		"""
        if const.appcfg.debug:
            print(f'Network ID: {net_id} - {net_name}')
        NetworkWirelessSsidIdentityPsks = []
        for ssid_num in range(0, 15):
            if ssid_num in self.ssids_range:
                NetworkWirelessSsidIdentityPsks.append(
                    await sdk.wireless.getNetworkWirelessSsidIdentityPsks(
                        net_id, ssid_num))
            else:
                NetworkWirelessSsidIdentityPsks.append([])
        self.NetworkWirelessSsidIdentityPsks = NetworkWirelessSsidIdentityPsks

    async def Update_NetworkWirelessSsid(self, sdk: object, org_id: str,
                                         net_id: str, net_name):
        # TODO Function needs to be refactored
        """
		Compaires Wireless Network SSID Configuration to Master and Updates
		SSIDs if anything changes on target network
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:
			Nothing Updates meraki Dashbioard
		"""
        # SSIDS
        # Process all SSIDs
        if const.appcfg.debug:
            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
        golden = lib.get_golden('wireless')
        for i in range(0, 15):
            # Don't process SSIDs that are unconfigured
            if 'Unconfigured SSID' in self.NetworkWirelessSsid[i][
             'name'] \
              and 'Unconfigured SSID' in \
              golden.NetworkWirelessSsid[i]['name']:
                continue
            temp_golden = copy.deepcopy(golden.NetworkWirelessSsid[i])
            temp_self = copy.deepcopy(self.NetworkWirelessSsid[i])
            override = False
            if 'name' in temp_golden:
                for ssid in const.appcfg.ssid_skip_psk:
                    if ('psk' in temp_golden) and (ssid
                                                   == temp_golden['name']):
                        override = True
                    if (temp_self['name'] == ssid) and \
                      ('psk' in temp_golden) and (
                      'psk' in temp_self):
                        temp_golden.pop('psk')
                        temp_self.pop('psk')
                        override = False
                    elif (temp_self['name'] == ssid) and \
                      ('psk' in temp_golden):
                        temp_golden.pop('psk')
                        override = False

            if not await lib.soft_compare(temp_golden, temp_self):
                # Make a copy of the golden SSID.... overrides will be needed to write
                print(
                    f'\t-{lib.bc.WARNING} {net_name} = {lib.bc.OKBLUE} SSID_Num[{i}] configuring '
                    f'SSID[{temp_golden["name"]}] ')

                # START OF THE OVERRIDES/EXCEPTIONS
                if 'encryptionMode' in temp_golden and temp_golden[
                        'encryptionMode'] == 'wpa-eap':
                    temp_golden['encryptionMode'] = 'wpa'
                # If the SSID has a single radius server,
                # it'll error if these are set to "None" so pop them
                if 'radiusFailoverPolicy' in temp_golden and \
                  temp_golden['radiusFailoverPolicy'] is None:
                    temp_golden.pop('radiusFailoverPolicy')
                    # temp_SSID['radiusFailoverPolicy'] = 'Allow access'
                if 'radiusLoadBalancingPolicy' in temp_golden and \
                  temp_golden['radiusLoadBalancingPolicy'] is None:
                    temp_golden.pop('radiusLoadBalancingPolicy')
                # this is to fix the case where the "target" network has
                # APvlanTags but the source does not. This wipes the target
                # if the source has no tags.
                if 'apTagsAndVlanIds' not in temp_golden:
                    temp_golden['apTagsAndVlanIds'] = []

                if 'radiusServers' in temp_golden:
                    for radius_server in temp_golden['radiusServers']:
                        radius_server['secret'] = const.appcfg.rad_keys_all

                if 'radiusAccountingServers' in temp_golden:
                    for ras in temp_golden['radiusAccountingServers']:
                        ras['secret'] = const.appcfg.rad_keys_all
                if override:
                    if self.overridePSK is not None:
                        temp_golden['psk'] = self.overridePSK

                # END OF THE OVERRIDES/EXCEPTIONS
                if const.appcfg.write:
                    lib.print_update(net_name, "SSID's", "Wireless")
                    await sdk.wireless.updateNetworkWirelessSsid(
                        net_id, **temp_golden)
                    await self.Get_NetworkWirelessSsid(sdk, net_id, net_name)

            else:
                lib.print_matched(net_name, "SSID's", "Wireless")

    async def Update_NetworkWirelessSsidFirewallL3FirewallRules(
            self, sdk: object, org_id: str, net_id: str, net_name):
        """
		Updates L3 firewall rules on the SSIDs
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:

		"""
        if const.appcfg.debug:
            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
        golden = lib.get_golden('wireless')
        # TODO Simplify logic
        for ssid in self.NetworkWirelessSsid:
            i = ssid['number']
            if 'Unconfigured SSID' in self.NetworkWirelessSsid[i][
             'name'] or 'Unconfigured SSID' in \
              golden.NetworkWirelessSsid[
               i]['name']:
                pass
            else:
                if not await lib.compare(
                        self.NetworkWirelessSsidFirewallL3FirewallRules[i],
                        golden.NetworkWirelessSsidFirewallL3FirewallRules[i]):
                    # print(f'L3 is not the same')
                    print(f'\t\t-{lib.bc.OKBLUE} Copied L3 rules for SSID['
                          f'{self.NetworkWirelessSsid[i]["name"]}] ')
                    lan_access = True
                    l3rules = copy.deepcopy(
                        golden.NetworkWirelessSsidFirewallL3FirewallRules[i])
                    new_l3 = {'rules': []}
                    for rule in l3rules['rules']:
                        if rule['destCidr'] == "Local LAN":
                            if rule['policy'] == "deny":
                                lan_access = False
                            else:
                                lan_access = True
                            # pull out the allow Lan Access rule, it's boolean
                            l3rules['rules'].remove(rule)
                        # pull out default rule, always the same
                        if rule['comment'] == "Default rule" \
                          or not rule['destCidr'] == "Local LAN":
                            new_l3['rules'].append(rule)

                    # print(f'L3 Rules are {newL3}')
                    new_l3['allowLanAccess'] = lan_access
                    if const.appcfg.write:
                        lib.print_update(net_name, "Layer3 Firewall Rules",
                                         "Wireless")
                        self.NetworkWirelessSsidFirewallL3FirewallRules[i] \
                         = await sdk.wireless.updateNetworkWirelessSsidFirewallL3FirewallRules(
                          net_id, i, **new_l3)
                        await self.Get_NetworkWirelessSsidFirewallL3FirewallRules(
                            sdk, net_id, net_name)
                else:
                    lib.print_matched(net_name, "Layer3 Firewall Rules",
                                      "Wireless")

    async def Update_NetworkWirelessSsidFirewallL7FirewallRules(
            self, sdk: object, org_id: str, net_id: str, net_name):
        """
		Updates Layer7 Firewall rules on the SSIDs
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:

		"""
        if const.appcfg.debug:
            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
        golden = lib.get_golden('wireless')
        for ssid in self.NetworkWirelessSsid:
            i = ssid['number']
            if 'Unconfigured SSID' in self.NetworkWirelessSsid[i][
             'name'] \
              or 'Unconfigured SSID' in \
              golden.NetworkWirelessSsid[i]['name']:
                continue
            if not await lib.compare(
                    self.NetworkWirelessSsidFirewallL7FirewallRules[i],
                    golden.NetworkWirelessSsidFirewallL7FirewallRules[i]):
                l7rules = \
                 copy.deepcopy(
                   golden.NetworkWirelessSsidFirewallL7FirewallRules[
                    i])
                # print(f'L7 not the same ... cloning')
                print(f'\t\t-{lib.bc.OKBLUE} '
                      f'Copied L7 rules for '
                      f'SSID[{self.NetworkWirelessSsid[i]["name"]}] ')

                if const.appcfg.write:
                    lib.print_update(net_name, "Layer7 Firewall Rules",
                                     "Wireless")
                    self.NetworkWirelessSsidFirewallL7FirewallRules[i] = \
                     await sdk.wireless.updateNetworkWirelessSsidFirewallL7FirewallRules(
                       net_id, i, **l7rules)
                    await self.Get_NetworkWirelessSsidFirewallL7FirewallRules(
                        sdk, net_id, net_name)

            else:
                lib.print_matched(net_name, "Layer7 Firewall Rules",
                                  "Wireless")

    async def Update_NetworkWirelessSsidTrafficShapingRules(
            self, sdk: object, org_id: str, net_id: str, net_name):
        """
		Updates Traffic Shaping Rules on the SSIDs
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:

		"""
        if const.appcfg.debug:
            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
        golden = lib.get_golden('wireless')
        for ssid in self.NetworkWirelessSsid:
            i = ssid['number']
            if 'Unconfigured SSID' in self.NetworkWirelessSsid[i][
             'name'] \
              or 'Unconfigured SSID' in \
              golden.NetworkWirelessSsid[i]['name']:
                continue
            if not await lib.compare(
                    self.NetworkWirelessSsidTrafficShapingRules[i],
                    golden.NetworkWirelessSsidTrafficShapingRules[i]):
                print(
                    f'\t\t-{lib.bc.OKBLUE} Copied Traffic '
                    f'Shaping rules for SSID[{self.NetworkWirelessSsid[i]["name"]}] '
                )
                try:
                    ts_rules = copy.deepcopy(
                        golden.NetworkWirelessSsidTrafficShapingRules[i])
                    if const.appcfg.write:
                        lib.print_update(net_name, "Traffic Shaping Rules",
                                         "Wireless")
                        self.NetworkWirelessSsidTrafficShapingRules[i] \
                         = await sdk.wireless.updateNetworkWirelessSsidTrafficShapingRules(
                          net_id, i, **ts_rules)
                        await self.Get_NetworkWirelessSsidTrafficShapingRules(
                            sdk, net_id, net_name)
                except Exception as error:
                    print(f'\t\t-{lib.bc.FAIL}Failed to update TrafficShaping.'
                          f' Make sure all rules are complete{lib.bc.ENDC} '
                          f'Error Code: {str(error)}')
            else:
                lib.print_matched(net_name, "Traffic Shaping Rules",
                                  "Wireless")
        # this also updates ssids_range

    async def Update_aironetie(self, sdk: object, org_id: str, net_id: str,
                               net_name):
        #TODO Need to check why funciton is throwing an error
        """
		Updates AiroNet IE Settins on the Meraki Dashboard
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:

		"""
        pass
#        if const.appcfg.debug:
#            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
#        golden = lib.get_golden('wireless')
        #for i in self.ssids_range:  # and self.hasAironetIE:
        #    if 'Unconfigured SSID' in self.NetworkWirelessSsid[i][
        #     'name'] \
        #      or 'Unconfigured SSID' in \
        #      golden.NetworkWirelessSsid[i]['name']:
        #        continue
        #    if self.hasAironetIE and not await lib.compare(
        #            self.aironetie[i], golden.aironetie[i]):
        #        if const.appcfg.write:
        #            await lib.set_aironet_ie(net_id, i, golden.aironetie[i])
        #            print(
        #                f'{lib.bc.OKBLUE}\t\tConfiguring AironetIE{lib.bc.WARNING}'
        #                f'{golden.aironetie[i]}'
        #                f'{lib.bc.OKBLUE}] on SSID[{lib.bc.WARNING}'
        #                f'{i}{lib.bc.OKBLUE}]{lib.bc.ENDC}')
        #            if self.hasAironetIE:
        #                await self.Get_aironetie(sdk, net_id, net_name)
        #    else:
        #        lib.print_matched(net_name, "AironetIE", "Wireless")

    async def Update_NetworkWirelessRfProfiles(self, sdk: object, org_id: str,
                                               net_id: str, net_name):
        # TODO Function needs to be refactored
        """
		Compaires Wireless Network RF Profiles with the Master Configuratio
		and Updates The RF Profile If neede
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:
			Nothing Updates meraki Dashbioard
		"""
        if const.appcfg.debug:
            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
        # RFProfiles - (if it exists and not equal, delete/update. If it doesn't exist, create)
        golden = lib.get_golden('wireless')
        resync = False
        self_rfps = copy.deepcopy(self.NetworkWirelessRfProfiles)
        golden_rfps = copy.deepcopy(golden.NetworkWirelessRfProfiles)
        self_rfps = lib.rf_profile_pre_proccess(self_rfps)
        golden_rfps = lib.rf_profile_pre_proccess(golden_rfps)

        if not await lib.compare(self_rfps, golden_rfps):  # Profiles are NOT the same
            for golden_profile in golden_rfps:
                rf_id = lib.find_rf_profile_id_by_name(golden_profile['bame'],self_rfps)
                profile_update = copy.deepcopy(golden_profile)
                profile_update = lib.add_valid_channels(profile_update)
                profile_update = await lib.rfp_pwr(profile_update)
          
                if rf_id is None:
                    print(
                        f'\t{lib.bc.OKBLUE}RF Profile[{lib.bc.WARNING}{golden_profile["name"]}'
                        f'{lib.bc.OKBLUE}]!!! New RFP created in network{lib.bc.ENDC}')
                    if const.appcfg.write:
                        try:
                            resync = True
                            await sdk.wireless.createNetworkWirelessRfProfile(net_id,**profile_update)
                        except AsyncAPIError as error:
                            print(str(error))
                        except Exception as error:
                            print(str(error))
                else:
                    print(f'{lib.bc.WARNING}{net_name}\t'
                        f'{lib.bc.OKBLUE}RF Profile[{lib.bc.WARNING}'
                        f'{golden_profile["name"]}{lib.bc.OKBLUE}] Not Found'
                         f'!!! Updating RF Profile{lib.bc.ENDC}')
                    if const.appcfg.write:
                        try:
                            resync = True
                            await sdk.wireless.updateNetworkWirelessRfProfile(
                                 net_id, rf_id, **profile_update)
                        except AsyncAPIError as error:
                             print(str(error))
                        except Exception as error:
                             print(str(error))
                for profile in self_rfps:
                    if not lib.find_rf_profile(profile['name'],golden_profile):
                        if const.appcfg.write:
                            try:
                                resync = True
                                await sdk.wireless.deleteNetworkWirelessRfProfile(
                                     net_id, profile['id'])
                            except AsyncAPIError as error:
                                 print(str(error))
                            except Exception as error:
                                 print(str(error))
        if resync:
                await re_sync(sdk, org_id, net_id, "NetworkWirelessRfProfiles",
                  "wireless")

    async def Update_NetworkWirelessSsidIdentityPsks(self, sdk: object,
                                                     org_id: str, net_id: str,
                                                     net_name):
        # TODO Function needs to be refactored
        """
		Compaires SSID iPSKs with the Master Configuratio
		and Updates The RF Profile If neede
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:
			Nothing Updates meraki Dashbioard
		"""
        if const.appcfg.debug:
            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
        # NetworkWirelessSsidIdentityPsks
        golden = lib.get_golden('wireless')
        golden_group_ploicy = lib.get_golden_group_policy()
        group_policy = lib.get_network_group_policy(org_id, net_id)
        resync = False
        ipsk_tmp = []
        for id_number in range(0, 15):
            ipsk_tmp.append({id_number})
        for ssid_num in self.ssids_range:
            # if not ssid_num in self.ssids_range: continue
            # ipsk_tmp.append({}) #keep track of golden iPSKs so we can
            # remove unused ones from local(self)
            for m_ipsk in golden.NetworkWirelessSsidIdentityPsks[ssid_num]:
                if not m_ipsk['name'] in ipsk_tmp[ssid_num]:
                    ipsk_tmp[ssid_num][m_ipsk['name']] = m_ipsk['passphrase']

                # ipsks are not empty, find the matching group policy
                new_ipsk = copy.deepcopy(m_ipsk)
                # pop off the ID from golden, new one will be created "local"
                new_ipsk.pop('net_id')
                golden_gp_tmp = await lib.matchGidByName(
                    golden_group_ploicy, str(new_ipsk['groupPolicyId']))
                local_gp_tmp = await lib.idFromName(group_policy,
                                                    str(golden_gp_tmp['name']))
                new_ipsk['groupPolicyId'] = local_gp_tmp['groupPolicyId']

                for s_ipsk in self.NetworkWirelessSsidIdentityPsks[ssid_num]:
                    if new_ipsk['name'] == s_ipsk['name']:
                        # if passwords are different, delete the ipsk and re-create
                        if new_ipsk['passphrase'] != s_ipsk['passphrase']:
                            if const.appcfg.write:
                                resync = True
                                try:
                                    await sdk.wireless.deleteNetworkWirelessSsidIdentityPsk(
                                        net_id, ssid_num, s_ipsk['net_id'])
                                except Exception as error:
                                    print(f'ERROR: iPSK Issue, resyncing and '
                                          f'trying again Error: {str(error)}')
                                    await self.Get_NetworkWirelessSsidIdentityPsks(
                                        sdk, net_id, net_name)
                                    await sdk.wireless.deleteNetworkWirelessSsidIdentityPsk(
                                        net_id, ssid_num, s_ipsk['id'])
                                    resync = True

                    else:
                        try:
                            await sdk.wireless.createNetworkWirelessSsidIdentityPsk(
                                net_id, ssid_num, **new_ipsk)
                            resync = True
                        except Exception as error:
                            print(
                                f'{lib.bc.FAIL}{net_name}'
                                f' - Idenity PSK \t\t{lib.bc.FAIL} '
                                f'iPSK already created or still there Error: '
                                f'{str(error)}{lib.bc.ENDC}')

        if resync:
            await self.Get_NetworkWirelessSsidIdentityPsks(
                sdk, net_id, net_name)

        # cleanUP local iPSK
        for ssid_num in self.ssids_range:
            for s_ipsk in self.NetworkWirelessSsidIdentityPsks[ssid_num]:
                if not s_ipsk['name'] in ipsk_tmp[ssid_num]:
                    if const.appcfg.write:
                        print(f'\t\t{lib.bc.OKBLUE}-Removing Legacy iPSK['
                              f'{s_ipsk["name"]}]{lib.bc.ENDC}')
                        await sdk.wireless.deleteNetworkWirelessSsidIdentityPsk(
                            net_id, ssid_num, s_ipsk['net_id'])

    async def Update_overridePSK(self, sdk: object, org_id: str, net_id: str,
                                 net_name):
        """
		Updates teh PSKs on the OVerRide List
		Args:
			sdk: Meraki SDK Object
			org_id: Current Org ID
			net_id: Curremt Metwork ID
			net_name: Network Name
		Returns:
		"""
        resync = False
        if const.appcfg.debug:
            print(f'ORG ID: {org_id} - Network ID: {net_id} - {net_name}')
        if const.appcfg.usepsk_file and (const.appcfg.psk_file is not None) \
          and (self.overridePSK is not None):
            for ssid in self.NetworkWirelessSsid:
                if ssid['name'] in const.appcfg.ssid_skip_psk:
                    if 'psk' in ssid:
                        if not await lib.compare(ssid['psk'],
                                                 self.overridePSK):
                            ssid['psk'] = self.overridePSK
                            if const.appcfg.write:
                                resync = True
                                lib.print_update(net_name,
                                                 "Site PSK Did not Match CSV",
                                                 "wireless")
                                await sdk.wireless.updateNetworkWirelessSsid(
                                    net_id, **ssid)
        if resync:
            await self.Get_NetworkWirelessSsid(sdk, net_id, net_name)



from .appconfig import APPCONFIG
from .mnet import MNET
from .orgs import ORGDB
from .meraki import meraki_nets, golden_nets,config_manager
from .device import Device
from .config_tag_manager import Tag_Manager
